
#include "k_inc_generic" 

// check if the player has a lightsaber
int hasLightsaber()
{
	int hasSaber = FALSE;
   int itemBaseType;

	if(feedback)
		SendMessageToPC(GetFirstPC(), "hasLightsaber: Started");

   // get the first inventory item
   object oInvItem = GetFirstItemInInventory(GetFirstPC());

   while(GetIsObjectValid(oInvItem))
   {
		itemBaseType = GetBaseItemType(oInvItem);

		// determine if this item is a lightsaber
      if((itemBaseType == BASE_ITEM_LIGHTSABER) ||
         (itemBaseType == BASE_ITEM_DOUBLE_BLADED_LIGHTSABER) ||
      	(itemBaseType == BASE_ITEM_SHORT_LIGHTSABER))
      {
         hasSaber = TRUE;
			if(feedback)
				SendMessageToPC(GetFirstPC(), "hasLightsaber: Lightsaber found in inventory");
         break;
      }

      oInvItem =  GetNextItemInInventory(GetFirstPC());
	}
			
	// if a lightsaber hasn't been found in the inventory
	// check to see if there is one equiped
	if(hasSaber != TRUE)
	{
		// haven't found lightsaber yet
		// looping over the entire slot range isn't necessary
		// but it makes things a little cleaner
		int slot;
		for(slot=0; slot < NUM_INVENTORY_SLOTS; slot++)
		{		
			oInvItem = GetItemInSlot(slot, GetFirstPC());
			if(GetIsObjectValid(oInvItem))
			{			  
				// determine if the equiped item is a lightsaber
				itemBaseType = GetBaseItemType(oInvItem);

        		if((itemBaseType == BASE_ITEM_LIGHTSABER) ||
            	(itemBaseType == BASE_ITEM_DOUBLE_BLADED_LIGHTSABER) ||
            	(itemBaseType == BASE_ITEM_SHORT_LIGHTSABER))
        		{  
					// found a lightsaber
            	hasSaber = TRUE;
					if(feedback)
						SendMessageToPC(GetFirstPC(), "hasLightsaber: Lightsaber found equiped");
            	break;
        		}
			}
 		}
	}

   return hasSaber;
}


// give the NPC some dark side force powers
// Note: it appears that only Horror and Insanity are used ??
void giveDSForcePowers(object oNPC)
{
	if(feedback)
		SendMessageToPC(GetFirstPC(), "giveDSForcePowers: Granting force powers");

	// give some force powers
	if(Random(forceRatio) == 0)
	{
   	GrantSpell(FORCE_POWER_LIGHTNING, oNPC);
   	GrantSpell(FORCE_POWER_DRAIN_LIFE, oNPC);
   	GrantSpell(FORCE_POWER_FORCE_WHIRLWIND, oNPC);
		GrantSpell(FORCE_POWER_INSANITY, oNPC);
   	GrantSpell(FORCE_POWER_HORROR, oNPC);
   	GrantSpell(FORCE_POWER_FORCE_PUSH, oNPC);
   	GrantSpell(FORCE_POWER_SHOCK, oNPC);
   	GrantSpell(FORCE_POWER_CHOKE, oNPC);
	}
}


// give a lightsaber and required feats to the NPC
void giveLightsaber(object oCreature, int saberType)
{
	// to use a lightsaber the NPC needs some melee and lightsaber feats
	if(GetFeatAcquired(FEAT_WEAPON_PROFICIENCY_MELEE_WEAPONS, oCreature) == FALSE)
 		GrantFeat(FEAT_WEAPON_PROFICIENCY_MELEE_WEAPONS, oCreature);
	if(GetFeatAcquired(FEAT_DOUBLE_WEAPON_FIGHTING, oCreature) == FALSE)
   	GrantFeat(FEAT_DOUBLE_WEAPON_FIGHTING, oCreature);
	if(GetFeatAcquired(FEAT_WEAPON_PROFICIENCY_LIGHTSABER, oCreature) == FALSE)
 		GrantFeat(FEAT_WEAPON_PROFICIENCY_LIGHTSABER, oCreature);
	if(GetFeatAcquired(FEAT_WEAPON_FOCUS_LIGHTSABER, oCreature) == FALSE)
   	GrantFeat(FEAT_WEAPON_FOCUS_LIGHTSABER, oCreature);

	object oSaber, oSaber2;
	switch(saberType)
	{
		case 0: 		// single lightsaber SINGLESABER	
			// give the NPC the lightsaber
   		oSaber = CreateItemOnObject(singleSaber, oCreature);

   		// have the NPC equip the lightsaber
   		AssignCommand(oCreature, ActionEquipItem(oSaber, INVENTORY_SLOT_RIGHTWEAPON));

			if(feedback)
				SendMessageToPC(GetFirstPC(), "giveLightsaber: single lightsaber equiped");
			break;
		case 1: 		// double lightsaber DOUBLESABER
			// give the NPC the lightsaber
   		oSaber = CreateItemOnObject(doubleSaber, oCreature);

   		// have the NPC equip the lightsaber
   		AssignCommand(oCreature, ActionEquipItem(oSaber, INVENTORY_SLOT_RIGHTWEAPON));

			if(feedback)
				SendMessageToPC(GetFirstPC(), "giveLightsaber: double lightsaber equiped");
			break;
		case 2:		// dual lightsaber DUALSABER
			// give the NPC the lightsabers
			oSaber = CreateItemOnObject(singleSaber, oCreature);
			oSaber2 = CreateItemOnObject(dualSaber, oCreature);	

   		// have the NPC equip the lightsaber
			AssignCommand(oCreature, ActionEquipItem(oSaber, INVENTORY_SLOT_RIGHTWEAPON));
			AssignCommand(oCreature, ActionEquipItem(oSaber2, INVENTORY_SLOT_LEFTWEAPON));
		
			if(feedback)
				SendMessageToPC(GetFirstPC(), "giveLightsaber: dual lightsabers equiped");
			break;
		default:
			break;
  	}
}


// give the NPC a melee weapon
void giveMeleeWeapon(object oNPC)
{
	// give the NPC a melee weapon
	object oMelee = CreateItemOnObject(meleeWeapon, oNPC);

	// grant the required melee feats
	if(GetFeatAcquired(FEAT_WEAPON_PROFICIENCY_MELEE_WEAPONS, oNPC) == FALSE)
 		GrantFeat(FEAT_WEAPON_PROFICIENCY_MELEE_WEAPONS, oNPC);
	if(GetFeatAcquired(FEAT_DOUBLE_WEAPON_FIGHTING, oNPC) == FALSE)
   	GrantFeat(FEAT_DOUBLE_WEAPON_FIGHTING, oNPC);

	// equip the weapon
	AssignCommand(oNPC, ActionEquipItem(oMelee, INVENTORY_SLOT_RIGHTWEAPON));

	// if using two weapons
	if(dualWield)
	{
		object oMelee2 = CreateItemOnObject(meleeWeapon2, oNPC);	
		AssignCommand(oNPC, ActionEquipItem(oMelee2, INVENTORY_SLOT_LEFTWEAPON));
	} 
}


// give the NPC a ranged weapon
void giveRangedWeapon(object oNPC)
{
	// set the NPC AI to use ranged weapons
	if(GetNPCAIStyle(oNPC) != NPC_AISTYLE_RANGED_ATTACK)
		SetNPCAIStyle(oNPC, NPC_AISTYLE_RANGED_ATTACK);	

	// grant the required ranged feats
	if(GetFeatAcquired(FEAT_WEAPON_PROFICIENCY_BLASTER, oNPC) == FALSE)
 		GrantFeat(FEAT_WEAPON_PROFICIENCY_BLASTER, oNPC);
	if(GetFeatAcquired(FEAT_WEAPON_PROFICIENCY_BLASTER_RIFLE, oNPC) == FALSE)
 		GrantFeat(FEAT_WEAPON_PROFICIENCY_BLASTER_RIFLE, oNPC);
	if(GetFeatAcquired(FEAT_WEAPON_PROFICIENCY_HEAVY_WEAPONS, oNPC) == FALSE)
 		GrantFeat(FEAT_WEAPON_PROFICIENCY_HEAVY_WEAPONS, oNPC);
	if(GetFeatAcquired(FEAT_DOUBLE_WEAPON_FIGHTING, oNPC) == FALSE)
   	GrantFeat(FEAT_DOUBLE_WEAPON_FIGHTING, oNPC);
	
	// add the weapon to the NPCs inventory
	object oGun = CreateItemOnObject(rangedWeapon, oNPC);
	
	// equip the weapon
	AssignCommand(oNPC, ActionEquipItem(oGun, INVENTORY_SLOT_RIGHTWEAPON));

	// if using two weapons
	if(dualWield)
	{
		object oGun2 = CreateItemOnObject(rangedWeapon2, oNPC);	
		AssignCommand(oNPC, ActionEquipItem(oGun2, INVENTORY_SLOT_LEFTWEAPON));
	}
}

// give the NPC the desired weapon
void giveWeapon(object oNPC)
{
	// unequip right hand weapon
	object oWeapon = GetItemInSlot(INVENTORY_SLOT_RIGHTWEAPON, oNPC);
	if(oWeapon != OBJECT_INVALID)
	{
		if(feedback)
			SendMessageToPC(GetFirstPC(), "changeNPC: Unequiping right weapon");
		AssignCommand(oNPC, ActionUnequipItem(oWeapon)); 
		DestroyObject(oWeapon);
		if(feedback)
			SendMessageToPC(GetFirstPC(), "changeNPC: Right weapon unequiped and destroyed");
	}
		
	// unequip left hand weapon
	oWeapon = GetItemInSlot(INVENTORY_SLOT_LEFTWEAPON, oNPC);
	if(oWeapon != OBJECT_INVALID)
	{
		if(feedback)
			SendMessageToPC(GetFirstPC(), "changeNPC: Unequiping left weapon");
		AssignCommand(oNPC, ActionUnequipItem(oWeapon)); 
		DestroyObject(oWeapon);
		if(feedback)
			SendMessageToPC(GetFirstPC(), "changeNPC: Left weapon unequiped and destroyed");
	}

	// give new weapon
	if(weaponType == 0) // melee	
		giveMeleeWeapon(oNPC);
	else if(weaponType == 1) // ranged
		giveRangedWeapon(oNPC);
	else if(weaponType == 2) // lightsaber
	{
		switch(Random(saberRatio))
		{
			case 0:
				giveLightsaber(oNPC, DOUBLESABER);
				break;
			case 1:
				giveLightsaber(oNPC, DUALSABER);
				break;
			default:
				giveLightsaber(oNPC, SINGLESABER);
				break;
  		}
	}
}


// change the NPC to the specified settings
void changeNPC(object oNPC)
{
	if(feedback)
		SendMessageToPC(GetFirstPC(), "changeNPC: Started");

	// give DS force powers if specified
	if(grantDarkSidePowers)
   	giveDSForcePowers(oNPC);

   // change the appearance specified type
	if(changeAppearance)
	{
   	if(!useAppearance2)
			ChangeObjectAppearance(oNPC, newAppearance1);
		else
		{
			int iType = Random(appearanceRatio);

			if(iType == 0)
   			ChangeObjectAppearance(oNPC, newAppearance2);  
   		else
   			ChangeObjectAppearance(oNPC, newAppearance1);  
		}
		if(feedback)
			SendMessageToPC(GetFirstPC(), "changeNPC: Finished changing appearance");
	}

	if(changeWeapon)
		giveWeapon(oNPC);
}


// converts some of the NPCs in the module
void NPCModuleConversion()
{
	int doConversion = TRUE;
	int NPCCount = 0;

	if(feedback)
		SendMessageToPC(GetFirstPC(), "NPCModuleConversion: Started");

  	// get the first NPC
  	object oCreature = GetFirstObjectInArea(GetArea(OBJECT_SELF), OBJECT_TYPE_CREATURE);

	// determine if the player has a lightsaber yet
	// if not, do not do the conversion
	if(doSaberCheck)
	{
		if(hasLightsaber())
		{
			doConversion = TRUE;
			if(feedback)
				SendMessageToPC(GetFirstPC(), "NPCModuleConversion: Player has a lightsaber");
		}
		else
		{
			doConversion = FALSE;
			if(feedback)
				SendMessageToPC(GetFirstPC(), "NPCModuleConversion: Player does not have a lightsaber");
		}
	}
	
  	// check that this NPC is valid
  	while(GetIsObjectValid(oCreature) && doConversion)
  	{
		if(feedback)
			SendMessageToPC(GetFirstPC(), "NPCModuleConversion: This NPC = " + GetName(oCreature));

    	// see if the NPC is of the desired type. 
    	if(GetAppearanceType(oCreature) == sourceNPC)
		{	
			// determine of this NPC should be altered										   
			if(NPCCount % newNPCRatio == 0)
			{
				if(feedback)
					SendMessageToPC(GetFirstPC(), "NPCModuleConversion: Calling changeNPC()");
				
				SetLocalBoolean(oCreature, SW_FLAG_BOSS_AI, TRUE);
				changeNPC(oCreature);
			}
			else if(feedback)
				SendMessageToPC(GetFirstPC(), "NPCModuleConversion: Not calling changeNPC()");
			
			NPCCount++;
		}				
    	oCreature = GetNextObjectInArea(GetArea(OBJECT_SELF), OBJECT_TYPE_CREATURE);
  	}

	if(feedback)
		SendMessageToPC(GetFirstPC(), "NPCModuleConversion: Complete");
}


// converts one NPC for a spawn script
void NPCConversion()
{
	int doConversion = TRUE;

	if(feedback)
		SendMessageToPC(GetFirstPC(), "NPCConversion: Started");

	// determine if the player has a lightsaber yet
	// if not, do not do the conversion
	if(doSaberCheck == TRUE)
	{
		if(hasLightsaber())
		{
			doConversion = TRUE;
			if(feedback)
				SendMessageToPC(GetFirstPC(), "NPCConversion: Player has a lightsaber");
		}
		else
		{
			doConversion = FALSE;
			if(feedback)
				SendMessageToPC(GetFirstPC(), "NPCConversion: Player does not have a lightsaber");
		}
	}
	
  	if(doConversion)
  	{
		if(feedback)
			SendMessageToPC(GetFirstPC(), "NPCConversion: This NPC = " + GetName(OBJECT_SELF));

    	// see if the NPC is of the desired type. 
    	if(GetAppearanceType(OBJECT_SELF) == sourceNPC)
		{													   
			if(feedback)
				SendMessageToPC(GetFirstPC(), "NPCConversion: Calling changeNPC()");
			
			// change the NPC 
			SetLocalBoolean(OBJECT_SELF, SW_FLAG_BOSS_AI, TRUE);
			changeNPC(OBJECT_SELF);
		}
		else if(feedback)
			SendMessageToPC(GetFirstPC(), "NPCConversion: Not calling changeNPC()");
  	}

	if(feedback)
		SendMessageToPC(GetFirstPC(), "NPCConversion: Complete");
}	  
